'''
Southern Light World
'''

__all__ = 'LOCATIONS',


LOCATIONS = {
    'South Light World': {
        'type': 'area',
        'link': {
            'South Dark World': [('and', [
                ('settings', 'inverted'), ('item', 'mirror')])],
            'Light Bottle Ocarina': [],
            "Link's House Entrance (E)": [],
            'Hyrule Castle': [],
            'Fairy Pond Under Rocks (LW) Entrance (E)': [('item', 'pegasus')],
            'Haunted Grove': [('item', 'shovel')],
            'Southern Chest Game Entrance (E)': [],
            'Kakariko': [],
            "Thieves' Town Surface": [('and', [
                ('settings', 'inverted'), ('item', 'mirror')])],
            'Frog Prison': [('and', [
                ('settings', 'inverted'), ('item', 'mirror')])],
            'Library Entrance (E)': [],
            "Quarrelling Brothers' House East Entrance (E)": [],
            'Cave Near Haunted Grove Entrance (E)': [
                ('settings', 'inverted'),
                ('glitch', 'major'),
                ('and', [
                    ('glitch', 'overworld'), ('item', 'pegasus')])],
            'Southern Shores': [],
            'Lake Hylia Fortune Teller Entrance (E)': [],
            'Lake Hylia': [('item', 'flippers')],
            'Lake Hylia Shop Entrance (E)': [],
            'East Light World': [],
            'Small Lake Hylia Island': [
                ('glitch', 'major'),
                ('and', [
                    ('glitch', 'overworld'), ('item', 'pegasus')])]},
        'visible': {'Running Game': [], 'Small Lake Hylia Island': []}
    },
    "Link's House Entrance (E)": {
        'type': 'entrance_unique', 'map': 'light', 'coord': (363, 456),
        'link': {
            'South Light World': [],
            "Link's House Entrance (I)": [('nosettings', 'inverted')],
            'Bomb Shop Entrance (I)': [('settings', 'inverted')]}
    },
    "Link's House Entrance (I)": {
        'type': 'interior',
        'link': {
            "Link's House Entrance (E)": [('nosettings', 'inverted')],
            'Bomb Shop Entrance (E)': [('settings', 'inverted')],
            "Link's House": [('rabbitbarrier', None)]}
    },
    "Link's House": {
        'type': 'chest', 'map': 'light', 'coord': (363, 448),
        'link': {
            "Link's House Entrance (I)": [],
            'Ocarina': [('and', [
                ('item', 'ocarina'), ('access', 'Kakariko')])]}
    },
    'Fairy Pond Under Rocks (LW) Entrance (E)': {
        'type': 'entrance', 'map': 'light', 'coord': (313, 433),
        'link': {
            'South Light World': [],
            'Fairy Pond Under Rocks (LW) Entrance (I)': []}
    },
    'Fairy Pond Under Rocks (LW) Entrance (I)': {
        'type': 'interior',
        'link': {
            'Fairy Pond Under Rocks (LW) Entrance (E)': []}
    },
    'Haunted Grove': {
        'type': 'item', 'map': 'light', 'coord': (190, 438),
        'link': {
            'South Light World': []}
    },
    'Southern Chest Game Entrance (E)': {
        'type': 'entrance', 'map': 'light', 'coord': (143, 465),
        'link': {
            'South Light World': [],
            'Southern Chest Game Entrance (I)': []}
    },
    'Southern Chest Game Entrance (I)': {
        'type': 'interior',
        'link': {
            'Southern Chest Game Entrance (E)': []}
    },
    'Library Entrance (E)': {
        'type': 'entrance_unique', 'map': 'light', 'coord': (104, 436),
        'link': {
            'South Light World': [],
            'Library Entrance (I)': []}
    },
    'Library Entrance (I)': {
        'type': 'interior',
        'link': {
            'Library Entrance (E)': [],
            'Library': [('item', 'pegasus')]},
        'visible': {'Library': []}
    },
    'Library': {
        'type': 'chest', 'map': 'light', 'coord': (104, 432),
        'link': {
            'Library Entrance (I)': []}
    },
    "Quarrelling Brothers' House East Entrance (E)": {
        'type': 'entrance_unique', 'map': 'light', 'coord': (93, 476),
        'link': {
            'South Light World': [],
            "Quarrelling Brothers' House East Entrance (I)": []}
    },
    "Quarrelling Brothers' House East Entrance (I)": {
        'type': 'interior',
        'link': {
            "Quarrelling Brothers' House East Entrance (E)": [],
            "Quarrelling Brothers' House West Entrance (I)": [
                ('item', 'bombs')]}
    },
    "Quarrelling Brothers' House West Entrance (I)": {
        'type': 'interior',
        'link': {
            "Quarrelling Brothers' House East Entrance (I)": [
                ('item', 'bombs')],
            "Quarrelling Brothers' House West Entrance (E)": []}
    },
    "Quarrelling Brothers' House West Entrance (E)": {
        'type': 'entrance_unique', 'map': 'light', 'coord': (73, 476),
        'link': {
            "Quarrelling Brothers' House West Entrance (I)": [],
            'Running Game': [('rabbitbarrier', None)]}
    },
    'Running Game': {
        'type': 'item', 'map': 'light', 'coord': (22, 463),
        'link': {
            "Quarrelling Brothers' House West Entrance (E)": [],
            'South Light World': []},
    },
    'Cave Near Haunted Grove Entrance (E)': {
        'type': 'entrance_unique', 'map': 'light', 'coord': (177, 546),
        'link': {
            'South Light World': [],
            'Cave Near Haunted Grove Entrance (I)': []}
    },
    'Cave Near Haunted Grove Entrance (I)': {
        'type': 'interior',
        'link': {
            'Cave Near Haunted Grove Entrance (E)': [],
            'Cave Near Haunted Grove': [('rabbitbarrier', None)]}
    },
    'Cave Near Haunted Grove': {
        'type': 'cave', 'map': 'light', 'coord': (177, 546),
        'link': {
            'Cave Near Haunted Grove Entrance (I)': []}
    },
    'Lake Hylia Fortune Teller Entrance (E)': {
        'type': 'entrance', 'map': 'light', 'coord': (433, 530),
        'link': {
            'South Light World': [],
            'Lake Hylia Fortune Teller Entrance (I)': []}
    },
    'Lake Hylia Fortune Teller Entrance (I)': {
        'type': 'interior',
        'link': {
            'Lake Hylia Fortune Teller Entrance (E)': []}
    },
    'Lake Hylia Shop Entrance (E)': {
        'type': 'entrance_shop', 'map': 'light', 'coord': (482, 508),
        'link': {
            'South Light World': [],
            'Lake Hylia Shop Entrance (I)': []}
    },
    'Lake Hylia Shop Entrance (I)': {
        'type': 'interior',
        'link': {
            'Lake Hylia Shop Entrance (E)': []}
    },

    'Southern Shores': {
        'type': 'area',
        'link': {
            'South Dark World': [('and', [
                ('settings', 'inverted'), ('item', 'mirror')])],
            'Light Bottle Ocarina': [],
            'South Light World': [],
            'South Portal (LW)': [('item', 'hammer')],
            'Witch Hut Waterway': [('item', 'flippers')],
            'Locksmith': [('access', 'Locked Chest')],
            'Desert Fairy Entrance (E)': [],
            'Bombos Stone Mountain': [
                ('settings', 'inverted'),
                ('glitch', 'major'),
                ('and', [
                    ('glitch', 'overworld'), ('item', 'pegasus')])],
            'Desert': [],
            'Rich Thief Entrance (E)': [('item', 'powerglove')],
            'Item Under Water': [('access', 'Water Drain')],
            'Water Drain Entrance (E)': [],
            'South Fairy Entrance (E)': [('item', 'bombs')],
            'Moldorm Cave Entrance (E)': [('item', 'bombs')],
            'Lake Hylia': [
                ('item', 'flippers'),
                ('and', [('glitch', 'overworld'), ('rabbitbarrier', None)])],
            'Poor Thief Entrance (E)': [('item', 'powerglove')],
            'Ice Cave Main Entrance Entrance (E)': [],
            'Ice Cave Secret Entrance Entrance (E)': [('item', 'bombs')],
            'Eastern Shores': [('and', [
                ('settings', 'inverted'), ('item', 'mirror')])]}
    },
    'South Portal (LW)': {
        'type': 'area',
        'link': {
            'South Portal (DW)': [('and', [
                ('settings', 'inverted'), ('item', 'mirror')])],
            'Southern Shores': [('item', 'hammer')],
            'South Portal': [('item', 'powerglove')]},
    },
    'South Portal': {
        'type': 'area',
        'link': {
            'South Portal (LW)': [('and', [
                ('settings', 'inverted'), ('state', 'rabbit')])],
            'South Portal (DW)': [('and', [
                ('nosettings', 'inverted'), ('state', 'rabbit')])]}
    },
    'Witch Hut Waterway': {
        'type': 'area',
        'link': {
            'Southern Shores': [],
            'Witch Hut River': []}
    },
    'Locksmith': {
        'type': 'chest', 'map': 'light', 'coord': (224, 594),
        'link': {
            'Southern Shores': []}
    },
    'Desert Fairy Entrance (E)': {
        'type': 'entrance', 'map': 'light', 'coord': (184, 590),
        'link': {
            'Southern Shores': [],
            'Desert Fairy Entrance (I)': []}
    },
    'Desert Fairy Entrance (I)': {
        'type': 'interior',
        'link': {
            'Desert Fairy Entrance (E)': []}
    },
    'Bombos Stone Mountain': {
        'type': 'area',
        'link': {
            'Southern Shores': [],
            'Desert': [],
            'Bombos Stone': [('mudora', 'take')]},
        'visible': {'Bombos Stone': [('item', 'mudora')]}
    },
    'Bombos Stone': {
        'type': 'item', 'map': 'light', 'coord': (145, 610),
        'link': {
            'Bombos Stone Mountain': []}
    },
    'Rich Thief Entrance (E)': {
        'type': 'entrance', 'map': 'light', 'coord': (207, 634),
        'link': {
            'Southern Shores': [],
            'Rich Thief Entrance (I)': []}
    },
    'Rich Thief Entrance (I)': {
        'type': 'interior',
        'link': {
            'Rich Thief Entrance (E)': []}
    },
    'Item Under Water': {
        'type': 'item', 'map': 'light', 'coord': (296, 616),
        'link': {
            'Southern Shores': []}
    },
    'Water Drain Entrance (E)': {
        'type': 'entrance_unique', 'map': 'light', 'coord': (311, 620),
        'link': {
            'Southern Shores': [],
            'Water Drain Entrance (I)': []}
    },
    'Water Drain Entrance (I)': {
        'type': 'interior',
        'link': {
            'Water Drain Entrance (E)': [],
            'Water Drain': [('rabbitbarrier', None)]}
    },
    'Water Drain': {
        'type': 'chest', 'map': 'light', 'coord': (311, 616),
        'link': {
            'Water Drain Entrance (I)': []},
    },
    'South Fairy Entrance (E)': {
        'type': 'entrance', 'map': 'light', 'coord': (396, 517),
        'link': {
            'Southern Shores': [],
            'South Fairy Entrance (I)': []}
    },
    'South Fairy Entrance (I)': {
        'type': 'interior',
        'link': {
            'South Fairy Entrance (E)': []}
    },
    'Moldorm Cave Entrance (E)': {
        'type': 'entrance_unique', 'map': 'light', 'coord': (433, 621),
        'link': {
            'Southern Shores': [],
            'Moldorm Cave Entrance (I)': []}
    },
    'Moldorm Cave Entrance (I)': {
        'type': 'interior',
        'link': {
            'Moldorm Cave Entrance (E)': [],
            'Moldorm Cave': [('rabbitbarrier', None)]}
    },
    'Moldorm Cave': {
        'type': 'cave', 'map': 'light', 'coord': (433, 621),
        'link': {
            'Moldorm Cave Entrance (I)': []}
    },
    'Poor Thief Entrance (E)': {
        'type': 'entrance', 'map': 'light', 'coord': (599, 525),
        'link': {
            'Southern Shores': [],
            'Poor Thief Entrance (I)': []}
    },
    'Poor Thief Entrance (I)': {
        'type': 'interior',
        'link': {
            'Poor Thief Entrance (E)': []}
    },
    'Ice Cave Main Entrance Entrance (E)': {
        'type': 'entrance', 'map': 'light', 'coord': (607, 511),
        'link': {
            'Southern Shores': [],
            'Ice Cave Main Entrance Entrance (I)': []}
    },
    'Ice Cave Main Entrance Entrance (I)': {
        'type': 'interior',
        'link': {
            'Ice Cave Main Entrance Entrance (E)': []}
    },
    'Ice Cave Secret Entrance Entrance (E)': {
        'type': 'entrance_unique', 'map': 'light', 'coord': (593, 511),
        'link': {
            'Southern Shores': [],
            'Ice Cave Secret Entrance Entrance (I)': []}
    },
    'Ice Cave Secret Entrance Entrance (I)': {
        'type': 'interior',
        'link': {
            'Ice Cave Secret Entrance Entrance (E)': [],
            'Ice Cave': []}
    },
    'Ice Cave': {
        'type': 'cave', 'map': 'light', 'coord': (593, 511),
        'link': {
            'Ice Cave Secret Entrance Entrance (I)': []}
    },

    'Lake Hylia': {
        'type': 'area',
        'link': {
            'Frozen Lake': [('and', [
                ('settings', 'inverted'), ('item', 'mirror')])],
            'Southern Shores': [],
            'Crossworld Waterway': [],
            'Zora River Waterway': [],
            'Large Lake Hylia Island': [],
            'Small Lake Hylia Island': [('settings', 'inverted')],
            'South Light World': [],
            'East Light World': [],
            'Under The Bridge': []},
        'visible': {'Small Lake Hylia Island': []}
    },
    'Zora River Waterway': {
        'type': 'area',
        'link': {
            'Lake Hylia': [],
            'Waterfall Lake': []}
    },
    'Under The Bridge': {
        'type': 'item', 'map': 'light', 'coord': (469, 462),
        'link': {
            'Lake Hylia': []}
    },
    'Small Lake Hylia Island': {
        'type': 'item', 'map': 'light', 'coord': (483, 552),
        'link': {
            'Frozen Lake': [('and', [
                ('settings', 'inverted'), ('item', 'mirror')])],
            'Lake Hylia': [('item', 'flippers')]}
    },
    'Large Lake Hylia Island': {
        'type': 'area',
        'link': {
            'Ice Palace Exterior': [('and', [
                ('settings', 'inverted'), ('item', 'mirror')])],
            'Ice Palace Portal': [('item', 'titansmitts')],
            'Fairy Queen Cave Entrance (E)': []},
    },
    'Ice Palace Portal': {
        'type': 'area',
        'link': {
            'Ice Palace Exterior': [('and', [
                ('nosettings', 'inverted'), ('state', 'rabbit')])],
            'Large Lake Hylia Island': [('and', [
                ('settings', 'inverted'), ('state', 'rabbit')])]}
    },
    'Fairy Queen Cave Entrance (E)': {
        'type': 'entrance_shop', 'map': 'light', 'coord': (526, 564),
        'link': {
            'Large Lake Hylia Island': [],
            'Fairy Queen Cave Entrance (I)': []}
    },
    'Fairy Queen Cave Entrance (I)': {
        'type': 'interior',
        'link': {
            'Fairy Queen Cave Entrance (E)': []}
    },
}



'''
Death Mountain
'''

__all__ = 'LOCATIONS',


LOCATIONS = {
    'West Lower Death Mountain': {
        'type': 'area',
        'link': {
            'West Lower Mount Hebra': [('and', [
                ('nosettings', 'inverted'), ('item', 'mirror')])],
            'Heart Rock': [('and', [
                ('nosettings', 'inverted'), ('item', 'mirror')])],
            'Dark Bottle Ocarina': [],
            'West Death Mountain Fairy Entrance (E)': [],
            'Cave of Byrna Entrance (E)': [],
            'Hebra Portal': [],
            'West Upper Death Mountain': [('settings', 'inverted')],
            'East Lower Death Mountain': [
                ('and', [
                    ('glitch', 'overworld'), ('item', 'pegasus'),
                    ('or', [
                        ('item', 'pearl'), ('item', 'hammer')])]),
                ('glitch', 'major')],
            'East Dark World': [('and', [
                ('glitch', 'overworld'),
                ('item', 'mirror'),
                ('item', 'pegasus'),
                ('or', [
                    ('item', 'sword'),
                    ('item', 'hookshot')])])]}
    },
    'West Death Mountain Fairy Entrance (E)': {
        'type': 'entrance', 'map': 'dark', 'coord': (264, 125),
        'link': {
            'West Lower Death Mountain': [],
            'West Death Mountain Fairy Entrance (I)': [
                ('nosettings', 'inverted')],
            'Hebra Ascent (Top) Entrance (I)': [('settings', 'inverted')]}
    },
    'West Death Mountain Fairy Entrance (I)': {
        'type': 'interior',
        'link': {
            'West Death Mountain Fairy Entrance (E)': [
                ('nosettings', 'inverted')],
            'Hebra Ascent (Top) Entrance (E)': [('settings', 'inverted')]}
    },
    'Cave of Byrna Entrance (E)': {
        'type': 'entrance_unique', 'map': 'dark', 'coord': (375, 96),
        'link': {
            'West Lower Death Mountain': [],
            'Cave of Byrna Entrance (I)': []}
    },
    'Cave of Byrna Entrance (I)': {
        'type': 'interior',
        'link': {
            'Cave of Byrna Entrance (E)': [],
            'Cave of Byrna': [('and', [
                ('item', 'hammer'),
                ('or', [
                    ('and', [
                        ('item', 'cape'), ('or', [
                            ('item', 'halfmagic'), ('item', 'bottle')])]),
                    ('item', 'byrna')]),
                ('item', 'powerglove')])]}
    },
    'Cave of Byrna': {
        'type': 'cave', 'map': 'dark', 'coord': (375, 96),
        'link': {
            'Cave of Byrna Entrance (I)': [('and', [
                ('item', 'hammer'),
                ('or', [
                    ('and', [
                        ('item', 'cape'), ('or', [
                            ('item', 'halfmagic'), ('item', 'bottle')])]),
                    ('item', 'byrna')]),
                ('item', 'powerglove')])]}
    },

    'West Upper Death Mountain': {
        'type': 'area',
        'link': {
            'West Upper Mount Hebra': [('and', [
                ('nosettings', 'inverted'), ('item', 'mirror')])],
            'West Lower Death Mountain': [],
            "Ganon's Tower Entrance (E)": [
                ('and', [
                    ('nosettings', 'inverted'), ('macro', 'ganonstower')]),
                ('settings', 'inverted')],
            'East Upper Death Mountain': []}
    },
    "Ganon's Tower Entrance (E)": {
        'type': 'entrance_dungeon', 'map': 'dark', 'coord': (367, 35),
        'link': {
            'West Upper Death Mountain': [],
            "Ganon's Tower Entrance (I)": [('nosettings', 'inverted')],
            'Castle Tower Entrance (I)': [('settings', 'inverted')]}
    },

    'East Lower Death Mountain': {
        'type': 'area',
        'link': {
            'East Lower Mount Hebra': [('and', [
                ('nosettings', 'inverted'), ('item', 'mirror')])],
            'Death Mountain Shop Entrance (E)': [],
            'Death Mountain Cave Bottom Entrance Entrance (E)': [],
            'East Hebra Portal': [('item', 'titansmitts')]}
    },
    'Death Mountain Shop Entrance (E)': {
        'type': 'entrance_shop', 'map': 'dark', 'coord': (571, 96),
        'link': {
            'East Lower Death Mountain': [],
            'Death Mountain Shop Entrance (I)': []}
    },
    'Death Mountain Shop Entrance (I)': {
        'type': 'interior',
        'link': {
            'Death Mountain Shop Entrance (E)': []}
    },
    'Death Mountain Cave Bottom Entrance Entrance (E)': {
        'type': 'entrance_unique', 'map': 'dark', 'coord': (557, 96),
        'link': {
            'East Lower Death Mountain': [],
            'Death Mountain Cave Bottom Entrance Entrance (I)': []}
    },
    'Death Mountain Cave Bottom Entrance Entrance (I)': {
        'type': 'interior',
        'link': {
            'Death Mountain Cave Bottom Entrance Entrance (E)': [],
            'Death Mountain Cave': [('rabbitbarrier', None)],
            'Death Mountain Cave Top Entrance Entrance (I)': []}
    },
    'Death Mountain Cave': {
        'type': 'cave', 'map': 'dark', 'coord': (557, 96),
        'link': {
            'Death Mountain Cave Top Entrance Entrance (I)': []}
    },
    'Death Mountain Cave Top Entrance Entrance (I)': {
        'type': 'interior',
        'link': {
            'Death Mountain Cave': [('rabbitbarrier', None)],
            'Death Mountain Cave Top Entrance Entrance (E)': []}
    },
    'Death Mountain Cave Top Entrance Entrance (E)': {
        'type': 'entrance_unique', 'map': 'dark', 'coord': (564, 41),
        'link': {
            'Death Mountain Cave Top Entrance Entrance (I)': [],
            'East Upper Death Mountain': []}
    },

    'East Upper Death Mountain': {
        'type': 'area',
        'link': {
            'East Upper Mount Hebra': [('and', [
                ('nosettings', 'inverted'), ('item', 'mirror')])],
            'Dark Bottle Ocarina': [],
            'West Upper Death Mountain': [],
            'East Lower Death Mountain': [],
            'Death Mountain Cave Top Entrance Entrance (E)': [],
            'Pit Cave Entrance (E)': [('item', 'powerglove')],
            'Turtle Rock Portal (DW)': [('settings', 'inverted')],
            'Turtle Rock Entrance Entrance (E)': [
                ('access', 'Open Turtle Rock')],
            'Pit Cave Peak': [('and', [
                ('glitch', 'overworld'), ('item', 'pegasus')])],
            'Turtle Rock Balcony': [
                ('and', [
                    ('glitch', 'overworld'),
                    ('or', [
                        ('item', 'mirror'), ('item', 'pegasus')])])],
            'Turtle Rock Fairy Exit': [
                ('and', [
                    ('glitch', 'overworld'),
                    ('or', [
                        ('item', 'mirror'), ('item', 'pegasus')])]),
                ('glitch', 'major')]}
    },
    'Pit Cave Entrance (E)': {
        'type': 'entrance_unique', 'map': 'dark', 'coord': (546, 44),
        'link': {
            'East Upper Death Mountain': [],
            'Pit Cave Entrance (I)': []}
    },
    'Pit Cave Entrance (I)': {
        'type': 'interior',
        'link': {
            'Pit Cave Entrance (E)': [],
            'Pit Cave': []}
    },
    'Pit Cave': {
        'type': 'area',
        'link': {
            'Pit Cave Entrance (I)': [],
            'Pit Cave Pegasus Chest': [
                ('and', [
                    ('item', 'pegasus'), ('settings', 'placement_advanced')]),
                ('item', 'hookshot')],
            'Pit Cave Hookshot Chests': [('item', 'hookshot')],
            'Pit Cave Back Exit Entrance (I)': [('item', 'bombs')]}
    },
    'Pit Cave Pegasus Chest': {
        'type': 'drop', 'map': 'dark', 'coord': (546, 51),
        'link': {
            'Pit Cave': []}
    },
    'Pit Cave Hookshot Chests': {
        'type': 'drop', 'map': 'dark', 'coord': (546, 37),
        'link': {
            'Pit Cave': []}
    },
    'Pit Cave Back Exit Entrance (I)': {
        'type': 'interior',
        'link': {
            'Pit Cave': [('item', 'bombs')],
            'Pit Cave Back Exit Entrance (E)': []}
    },
    'Pit Cave Back Exit Entrance (E)': {
        'type': 'entrance_unique', 'map': 'dark', 'coord': (524, 10),
        'link': {
            'Pit Cave Back Exit Entrance (I)': [],
            'Pit Cave Peak': []}
    },
    'Pit Cave Peak': {
        'type': 'area',
        'link': {
            'Heart Peak': [('and', [
                ('nosettings', 'inverted'), ('item', 'mirror')])],
            'Pit Cave Back Exit Entrance (E)': [],
            'East Upper Death Mountain': []}
    },
    'Turtle Rock Portal (DW)': {
        'type': 'area',
        'link': {
            'Turtle Rock Portal (LW)': [('and', [
                ('nosettings', 'inverted'), ('item', 'mirror')])],
            'East Upper Death Mountain': [],
            'Turtle Rock Portal': [('item', 'titansmitts')],
            'Open Turtle Rock': [('medallion', 'Turtle Rock')]}
    },
    'Open Turtle Rock': {
        'type': 'area',
        'link': {
            'Turtle Rock Portal (DW)': []}
    },
}



'''
Eastern Light World
'''

__all__ = 'LOCATIONS',


LOCATIONS = {
    'East Light World': {
        'type': 'area',
        'link': {
            'River Shop Area': [('and', [
                ('settings', 'inverted'), ('item', 'mirror')])],
            'East Dark World': [('and', [
                ('settings', 'inverted'), ('item', 'mirror')])],
            'Light Bottle Ocarina': [],
            'Witch Hut River': [
                ('item', 'flippers'),
                ('and', [('glitch', 'overworld'), ('rabbitbarrier', None)])],
            'Witch Hut Entrance (E)': [('rabbitbarrier', None)],
            'Witch': [('rabbitbarrier', None)],
            "Zora's River": [
                ('item', 'powerglove'),
                ('and', [
                    ('glitch', 'overworld'), ('item', 'pegasus')])],
            'North Light World': [],
            'Hyrule Castle': [('item', 'powerglove')],
            'South Light World': [],
            'East Fairy Cave Entrance (E)': [],
            'Lake Hylia': [('item', 'flippers')],
            'East Fairy Pond Entrance (E)': [],
            'East Portal (LW)': [('item', 'hammer')],
            "Sahasrahla's Hideout Entrance (E)": [],
            'East Palace Entrance (E)': []}
    },
    'Witch Hut River': {
        'type': 'area',
        'link': {
            'Dark River': [('and', [
                ('settings', 'inverted'), ('item', 'mirror')])],
            'East Light World': [],
            'North Light World': [],
            'Witch Hut Waterway': []}
    },
    'Witch Hut Entrance (E)': {
        'type': 'entrance_unique', 'map': 'light', 'coord': (531, 224),
        'link': {
            'East Light World': [('rabbitbarrier', None)],
            'Witch Hut Entrance (I)': []}
    },
    'Witch Hut Entrance (I)': {
        'type': 'interior',
        'link': {
            'Witch Hut Entrance (E)': [],
            'Witch Hut': [('and', [('access', 'Witch'), ('item', 'mushroom')])]}
    },
    'Witch Hut': {
        'type': 'chest', 'map': 'light', 'coord': (531, 215),
        'link': {
            'Witch Hut Entrance (I)': []}
    },
    'Witch': {
        'type': 'area',
        'link': {
            'East Light World': [('rabbitbarrier', None)]}
    },
    'East Fairy Cave Entrance (E)': {
        'type': 'entrance', 'map': 'light', 'coord': (546, 428),
        'link': {
            'East Light World': [],
            'East Fairy Cave Entrance (I)': []}
    },
    'East Fairy Cave Entrance (I)': {
        'type': 'interior',
        'link': {
            'East Fairy Cave Entrance (E)': []}
    },
    'East Fairy Pond Entrance (E)': {
        'type': 'entrance', 'map': 'light', 'coord': (650, 464),
        'link': {
            'East Light World': [],
            'East Fairy Pond Entrance (I)': []}
    },
    'East Fairy Pond Entrance (I)': {
        'type': 'interior',
        'link': {
            'East Fairy Pond Entrance (E)': []}
    },
    'East Portal (LW)': {
        'type': 'area',
        'link': {
            'East Portal (DW)': [('and', [
                ('settings', 'inverted'), ('item', 'mirror')])],
            'East Light World': [('item', 'hammer')], 
            'East Portal': [('item', 'powerglove')]}
    },
    'East Portal': {
        'type': 'area',
        'link': {
            'East Portal (DW)': [('and', [
                ('nosettings', 'inverted'), ('state', 'rabbit')])],
            'East Portal (LW)': [('and', [
                ('settings', 'inverted'), ('state', 'rabbit')])]}
    },
    "Sahasrahla's Hideout Entrance (E)": {
        'type': 'entrance_unique', 'map': 'light', 'coord': (537, 300),
        'link': {
            'East Light World': [],
            "Sahasrahla's Hideout Entrance (I)": []}
    },
    "Sahasrahla's Hideout Entrance (I)": {
        'type': 'interior',
        'link': {
            "Sahasrahla's Hideout Entrance (E)": [],
            "Sahasrahla's Hideout": []}
    },
    "Sahasrahla's Hideout": {
        'type': 'area',
        'link': {
            "Sahasrahla's Hideout Entrance (I)": [],
            "Sahasrahla's Reward": [('pendant', 'courage')],
            "Sahasrahla's Treasure": [('item', 'bombs'), ('item', 'pegasus')]}
    },
    "Sahasrahla's Reward": {
        'type': 'chest', 'map': 'light', 'coord': (537, 300),
        'link': {
            "Sahasrahla's Hideout": []}
    },
    "Sahasrahla's Treasure": {
        'type': 'cave', 'map': 'light', 'coord': (537, 284),
        'link': {
            "Sahasrahla's Hideout": []}
    },

    "Zora's River": {
        'type': 'area',
        'link': {
            'Catfish Approach': [('and', [
                ('settings', 'inverted'), ('item', 'mirror')])],
            'East Light World': [('item', 'powerglove')],
            'Waterfall Lake': [('item', 'flippers')],
            'King Zora': [],
            'Zora River Viewpoint': [
                ('item', 'flippers'),
                ('and', [
                    ('glitch', 'overworld'), ('item', 'pegasus')])]},
        'visible': {'Zora River Viewpoint': []}
    },
    'Waterfall Lake': {
        'type': 'area',
        'link': {
            'Zora River Waterway': [],
            "Zora's River": [],
            'Waterfall Fairy Entrance (E)': [
                ('item', 'flippers'),
                ('and', [
                    ('glitch', 'overworld'),
                    ('or', [
                        ('item', 'pearl'), ('item', 'pegasus')])])]}
    },
    'Waterfall Fairy Entrance (E)': {
        'type': 'entrance_unique', 'map': 'light', 'coord': (597, 89),
        'link': {
            'Waterfall Lake': [('item', 'flippers')],
            'Waterfall Fairy Entrance (I)': []}
    },
    'Waterfall Fairy Entrance (I)': {
        'type': 'interior',
        'link': {
            'Waterfall Fairy Entrance (E)': [],
            'Waterfall Fairy': []}
    },
    'Waterfall Fairy': {
        'type': 'cave', 'map': 'light', 'coord': (597, 89),
        'link': {
            'Waterfall Fairy Entrance (E)': []}
    },
    'King Zora': {
        'type': 'item', 'map': 'light', 'coord': (640, 100),
        'link': {
            "Zora's River": []}
    },
    'Zora River Viewpoint': {
        'type': 'item', 'map': 'light', 'coord': (640, 120),
        'link': {
            "Zora's River": [('item', 'flippers')]}
    },

    'East Palace Entrance (E)': {
        'type': 'entrance_dungeon', 'map': 'light', 'coord': (636, 272),
        'link': {
            'East Palace Entrance (E)': [],
            'East Palace Entrance (I)': []}
    },
    'East Palace Entrance (I)': {
        'type': 'interior',
        'link': {
            'East Palace Entrance (E)': [],
            'East Palace Interior': [('rabbitbarrier', None)]}
    },
    'East Palace Interior': {
        'type': 'area', 'dungeon': 'East Palace',
        'link': {
            'East Palace Entrance (I)': [],
            'East Palace Rupee': [],
            'East Palace Map': [],
            'East Palace Compass': [],
            'East Palace Treasure': [('bigkey', 'East Palace')],
            'East Palace Dark Room': [('item', 'lantern')],
            'East Palace Boss Door': [('bigkey', 'East Palace')]}
    },
    'East Palace Rupee': {
        'type': 'dungeonchest', 'dungeon': 'East Palace',
        'link': {
            'East Palace Interior': []}
    },
    'East Palace Map': {
        'type': 'dungeonchest', 'dungeon': 'East Palace',
        'link': {
            'East Palace Interior': []}
    },
    'East Palace Compass': {
        'type': 'dungeonchest', 'dungeon': 'East Palace',
        'link': {
            'East Palace Interior': []}
    },
    'East Palace Treasure': {
        'type': 'dungeonchest_nokey', 'dungeon': 'East Palace',
        'link': {
            'East Palace Interior': []}
    },
    'East Palace Dark Room': {
        'type': 'area', 'dungeon': 'East Palace',
        'link': {
            'East Palace Interior': [],
            'East Palace Hidden Key': [],
            'East Palace Big Key': [('smallkey', ('East Palace', 0))]}
    },
    'East Palace Hidden Key': {
        'type': 'dungeonkey', 'dungeon': 'East Palace',
        'link': {
            'East Palace Dark Room': []}
    },
    'East Palace Big Key': {
        'type': 'dungeonchest', 'dungeon': 'East Palace',
        'link': {
            'East Palace Dark Room': []}
    },
    'East Palace Boss Door': {
        'type': 'area', 'dungeon': 'East Palace',
        'link': {
            'East Palace Interior': [],
            'East Palace Gauntlet Entrance': [
                ('item', 'lantern'),
                ('and', [
                    ('settings', 'placement_advanced'), ('item', 'firerod')])]}
    },
    'East Palace Gauntlet Entrance': {
        'type': 'area', 'dungeon': 'East Palace',
        'link': {
            'East Palace Boss Door': [],
            'East Palace Eyegore Key': [],
            'East Palace Gauntlet': [('smallkey', ('East Palace', 0))]}
    },
    'East Palace Eyegore Key': {
        'type': 'dungeonkey', 'dungeon': 'East Palace',
        'link': {
            'East Palace Gauntlet Entrance': []}
    },
    'East Palace Gauntlet': {
        'type': 'area', 'dungeon': 'East Palace',
        'link': {
            'East Palace Gauntlet Entrance': [
                ('item', 'lantern'),
                ('and', [
                    ('settings', 'placement_advanced'), ('item', 'firerod')])],
            'East Palace Boss': [('item', 'bow')]}
    },
    'East Palace Boss': {
        'type': 'dungeonboss', 'dungeon': 'East Palace',
        'link': {
            'East Palace Boss Item': [
                ('item', 'sword'), ('item', 'hammer'), ('item', 'bow'),
                ('item', 'boomerang'),
                ('and', [
                    ('or', [
                        ('item', 'bottle'), ('item', 'halfmagic')]),
                    ('or', [
                        ('item', 'firerod'), ('item', 'icerod'),
                        ('item', 'byrna'), ('item', 'somaria')])])]}
    },
    'East Palace Boss Item': {
        'type': 'dungeonchest_nokey', 'dungeon': 'East Palace',
        'link': {
            'East Palace Reward': []}
    },
    'East Palace Reward': {
        'type': 'dungeonreward', 'dungeon': 'East Palace',
        'link': {
            'East Palace Entrance (I)': []}
    },
}



'''
Entrance tracker
'''

from . import storage

__all__ = 'EntranceTracker',


class EntranceTracker(dict):
    '''
    Entrance tracker

    Instance variables:
        self: {'location': 'linked locations'}
        armed: True if entrance connection is armed
        worldtracker: world state tracker
    '''

    def __init__(self, worldtracker):
        '''
        Args:
            worldtracker: world state tracker
        '''

        super().__init__(storage.load_entrances())
        self.armed = False
        self.worldtracker = worldtracker

    def event(self, button: str) -> bool:
        '''
        Entrance connection event

        Connection happens in two steps: first it's armed, then it's connected.

        Args:
            button: name of location
        Returns:
            bool: True if new connection was made
        '''

        if ('retro' in self.worldtracker.settings and
            not 'entrance' in self.worldtracker.settings):
            return False
        if not self.armed:
            if button not in self:
                self.armed = button
            return False
        if button.replace(' (E)', ' (I)') in self.values():
            self.armed = False
            return False
        interior = button.replace(' (E)', ' (I)')
        self[self.armed] = interior
        self[interior] = self.armed
        self.armed = False
        self.save()
        return True

    def abort(self, button: str = None) -> bool:
        '''
        Abort or delete connection.

        Args:
            button: name of location
        '''

        if ('retro' in self.worldtracker.settings and
            not 'entrance' in self.worldtracker.settings):
            return False
        self.armed = False
        if button is not None and button in self:
            del self[self[button]]
            del self[button]
        self.save()

    def update(self) -> None:
        '''
        Update world tracker with new entrance information.
        '''

        self.worldtracker.ruleset.disconnect_entrances()
        for location in self:
            self.worldtracker.ruleset[location].link[self[location]] = []
            self.worldtracker.ruleset[self[location]].link[location] = []
        self.worldtracker.rulecheck()

    def reset(self) -> None:
        '''
        Remove all established connections.
        '''

        for loc in tuple(self.keys()):
            del self[loc]
        self.armed = False

    def find_remote(self, start: str) -> list:
        '''
        Find other interior conections.

        Args:
            start: starting entrance
        Returns:
            list: entrances connected via interiors
        '''

        if start not in self:
            return []
        reachable = [self[start]]
        done = {start}
        entrances = set()
        while reachable:
            loc = reachable.pop(0)
            done.add(loc)
            for link in self.worldtracker.ruleset[loc].link:
                if link in ('Ocarina', 'Pyramid', 'Castle Walls'):
                    continue
                if not self.worldtracker.ruleset[link].type.startswith(
                        'entrance') and link not in done:
                    reachable.append(link)
                elif self.worldtracker.ruleset[link].type.startswith(
                        'entrance') and link != start:
                    entrances.add(link)
        return entrances

    def save(self) -> None:
        '''
        Save current connections to file.
        '''

        storage.store_entrances(self)
